@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF8 -*-
 #
-# HabitJewel: Track your habits
+# Habit Jewel: Track your habits
 # Copyright (c) 2016 Ashley Hooper
 # <ashleyghooper@gmail.com>
 #
@@ -19,64 +19,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-VERSION = '0.5.0'
+VERSION = '0.7.1' # (major.minor.sub-minor)
 
-"""
-CHANGELOG:
-v0.5.0
-* Added New Habit button and 'clone habit' tap and hold option in MHL
-* Added habit parameters preview on habit editing screen
+# Minor version changes each time database schema changes 
+# See CHANGELOG.md for detailed change history
 
-v0.4.0
-* Added priority to the Edit Habits screen
-* Added null measure for habits which don't have any associated measure
-* Added a schema version history table, version will be inserted on database creation,
-  and future versions with incompatible schema changes can use this to determine the
-  version so as to know how to migrate to their newer schemas
-* Renamed habitjewel_utils.py to habitjewel_db.py
-* Moved database creation code to habitjewel_db.py and took out most of my sample habits
-* Changed frequency field/column to weekly quota/quota (easier to make sense of)
-* Fixed module path include code
-* Added timezone
-* Fixed portrait on startup using Khertan's patch to thp's FremantleRotation
-* Deleted cruft
-
-v0.3.0
-* Bumped version number due to incompatible schema changes
-
-v0.2.12
-* Fixed incorrect dependency on portrait for FremantleRotation
-* Changed habit schedules to based on weekly quota (repetitions) per weekly cycle
-
-v0.2.10
-* Changed back from Gtk UIManager to normal Gtk Menus for Hildon-styled popups
-
-v0.2.9
-* Fixed redraw of master habits list on orientation change
-* Redraw edit habit window when paused/unpaused or deleted/undeleted
-* Removed window stack, tried various ways to only handle rotation for visible window,
-  but failed so reverted to blanket rotation.
-* Created window stack to track top window to better handle rotation for windows
-* Make copy of habit object as editing_habit (and clean up on editing window destroy)
-* Added some date utility functions
-* Refactoring
-* Add DELETE & PAUSE to tap and hold menu for habits
-* Make master habit list, accessible from main menu to edit all habits (not just current day)
-
-TODO:
-* Test for orientation change issues (see FIXME)
-* Don't prepopulate habit table when no database - make placeholder for when empty instead
-* Restore single click on checkbox to toggle done/missed/unknown statuses
-* Move "About" page markup to external file
-* Track periods where habits were "paused" in order to calculate stats correctly
-* Add graphs
-* Visual indicator when browsing future or past days' habits
-* Move constants to a module
-* Move code at bare module scope into class(es)
-"""
-
-import datetime
+import cairo
 import calendar
+import datetime
 import fcntl
 import gettext
 import gobject
@@ -89,6 +39,10 @@ import sqlite3
 import sys
 import time
 
+# GStreamer for playing audio
+import pygst
+pygst.require('0.10')
+import gst
 
 # Get path to determine library and static resource locations
 running_path = sys.path[0]
@@ -100,6 +54,8 @@ import habitjewel_db
 
 
 # Constants
+APP_DISPLAY_NAME = 'Habit Jewel'
+APP_SYSTEM_NAME = 'habitjewel'
 WIN_PROG_IND = hildon.hildon_gtk_window_set_progress_indicator
 OSSO_CONTEXT = osso.Context('org.maemo.habitjewel', VERSION, False)
 
@@ -108,7 +64,8 @@ OSSO_CONTEXT = osso.Context('org.maemo.habitjewel', VERSION, False)
 LANDSCAPE, PORTRAIT = range(2)
 
 # Stackable Window titles
-WIN_TITLE_TOP                = 'Habit Jewel'
+WIN_TITLE_TOP                = APP_DISPLAY_NAME
+WIN_TITLE_TIMER              = 'Habit Jewel Timer'
 WIN_TITLE_GO_TO_DATE         = 'Go To Date'
 WIN_TITLE_PAUSE_UNTIL_DATE   = 'Pause Habit Until Date'
 WIN_TITLE_MASTER_HABITS_LIST = 'Master Habits List'
@@ -118,13 +75,17 @@ WIN_TITLE_ABOUT              = 'About HabitJewel'
 
 # Gtk CellRenderer wrapping widths
 # Day Habit List
-TV_DAY_ACTIVITY_WRAP_WIDTH_LANDSCAPE = 700
-TV_DAY_ACTIVITY_WRAP_WIDTH_PORTRAIT = 380
+TV_DAY_ACTIVITY_WRAP_WIDTH_LANDSCAPE = 650
+TV_DAY_ACTIVITY_WRAP_WIDTH_PORTRAIT  = 340
 # Master Habit List
 TV_MASTER_ACTIVITY_WRAP_WIDTH_LANDSCAPE = 550
-TV_MASTER_QUOTA_WRAP_WIDTH_LANDSCAPE = 250
-TV_MASTER_ACTIVITY_WRAP_WIDTH_PORTRAIT = 300
-TV_MASTER_QUOTA_WRAP_WIDTH_PORTRAIT = 170
+TV_MASTER_QUOTA_WRAP_WIDTH_LANDSCAPE    = 250
+TV_MASTER_ACTIVITY_WRAP_WIDTH_PORTRAIT  = 300
+TV_MASTER_QUOTA_WRAP_WIDTH_PORTRAIT     = 170
+
+# Timer activity display wrapping widths
+TIMER_ACTIVITY_WRAP_WIDTH_LANDSCAPE = 600
+TIMER_ACTIVITY_WRAP_WIDTH_PORTRAIT  = 300
 
 # TreeView constants
 # Day habits list treeview column indexes and titles
@@ -132,9 +93,11 @@ TV_DAY_COL_NUM_ID             = 0
 TV_DAY_COL_NAME_ID            = 'ID'
 TV_DAY_COL_NUM_ACTIVITY       = 1
 TV_DAY_COL_NAME_ACTIVITY      = 'Activity'
-TV_DAY_COL_NUM_CHECKBOX       = 2
+TV_DAY_COL_NUM_HISTICON       = 2
+TV_DAY_COL_NAME_HISTICON      = 'History'
+TV_DAY_COL_NUM_CHECKBOX       = 3
 TV_DAY_COL_NAME_CHECKBOX      = 'Checkbox'
-TV_DAY_COL_NUM_PCT_COMPLETE   = 3
+TV_DAY_COL_NUM_PCT_COMPLETE   = 4
 TV_DAY_COL_NAME_PCT_COMPLETE  = 'Percent complete'
 
 # Day habits list treeview column indexes and titles
@@ -165,35 +128,35 @@ HISTORY_MISSED_PIXBUF_FILE  = "checkbox_crossed.png"
 HISTORY_CLEAR_PCT           = -1
 HISTORY_CLEAR_PIXBUF_FILE   = "checkbox_unchecked.png"
 
+# Habit recent history graph size
+HABIT_RECENT_HIST_PIXBUF_SIZE = 48
+HABIT_RECENT_HIST_BAR_HEIGHT  = 32
+
 # Master Habits List status icons
-MASTER_STATUS_DELETED_PIXBUF_FILE = "habit_deleted.png"
-MASTER_STATUS_PAUSED_PIXBUF_FILE  = "habit_paused.png"
-MASTER_STATUS_ACTIVE_PIXBUF_FILE  = "habit_active.png"
+MASTER_STATUS_DELETED_PIXBUF_FILE = "generic_red_x_icon.png"
+MASTER_STATUS_PAUSED_PIXBUF_FILE  = "generic_pause_icon.png"
+MASTER_STATUS_ACTIVE_PIXBUF_FILE  = "generic_play_icon.png"
+
+# Timer icons
+TIMER_STOP_PIXBUF_FILE  = "generic_pause_icon.png"
+TIMER_START_PIXBUF_FILE = "generic_play_icon.png"
+
 
 # Misc constants
 NULL_MEASURE_DESC = '(None)'
-
+TIMER_TIMEOUT_INTERVAL_SECS = 10
 
 # Set timezone
 time.tzset()
 
 # User data initialisation
 home = os.path.expanduser('~')
-config_dir = home + '/.habitjewel/'
-db_file = config_dir + 'database'
-log_file = config_dir + 'log.txt'
+config_dir = home + '/.habitjewel'
 
-# Check for config dir, database. Create if necessary
+# Check for config dir
 if not os.path.exists(config_dir):
     os.mkdir(config_dir)
 
-if os.path.exists(db_file):
-    conn = sqlite3.connect(db_file)
-else:
-    conn = sqlite3.connect(db_file)
-    print 'creating new database'
-    habitjewel_db.create_new_database(conn)
-
 
 
 
@@ -205,14 +168,16 @@ else:
 class MainWindow:
 
     def __init__(self):
-        gettext.install('habitjewel','/opt/habitjewel/share/locale')
+        self.db = habitjewel_db.HabitJewelDb(config_dir, VERSION)
+
+        gettext.install(APP_SYSTEM_NAME,'/opt/habitjewel/share/locale')
 
         # Get today's date and use that as the date displayed on startup
         self.view_date_dt = self.get_today_dt()
 
         self.program = hildon.Program()
         self.program.__init__()
-        gtk.set_application_name('Habitjewel')
+        gtk.set_application_name(APP_DISPLAY_NAME)
 
         # Rotation setup
         orientation = self.get_current_screen_orientation()
@@ -223,7 +188,7 @@ class MainWindow:
         self.program.add_window(self.top_win)
 
         # N900-specific
-        self.osso_app_name = 'habitjewel'
+        self.osso_app_name = APP_SYSTEM_NAME
         self.rotation_obj = self.init_autorotation()
 
         # Determine current orientation
@@ -241,6 +206,9 @@ class MainWindow:
         self.top_win.set_app_menu(menu)
         self.setup_popup_menus()
 
+        # Initialise Timer
+        self.timer = {}
+
         self.top_container = self.get_day_habits_list_container()
         self.top_win.add(self.top_container)
 
@@ -346,11 +314,13 @@ class MainWindow:
             self.mhl_win.add(self.mhl_container)
             self.mhl_win.show_all()
 
-        # Rotation for Add/Edit Habit window
-        elif active_window_title == _(WIN_TITLE_ADD_NEW_HABIT) or \
-             active_window_title == _(WIN_TITLE_EDIT_HABIT):
+        # Rotation for Timer window
+        elif active_window_title == _(WIN_TITLE_TIMER):
             # No need to re-layout at present
-            return
+            self.timer_win.remove(self.timer_container)
+            self.timer_container = self.get_timer_container()
+            self.timer_win.add(self.timer_container)
+            self.timer_win.show_all()
 
 
 
@@ -362,14 +332,12 @@ class MainWindow:
 
 
     def set_db_habit_deleted_date(self, habit, deleted_date_dt):
-        habitjewel_db.delete_habit(conn, habit['id'])
+        self.db.delete_habit(habit['id'])
         self.show_info_banner(self.top_win, '"' + habit['activity'] + '" deleted')
 
 
     def set_db_habit_paused_until_date (self, habit, paused_until_date_dt):
-        habitjewel_db.set_habit_paused_until_date (conn,
-                habit['id'], paused_until_date_dt
-            )
+        self.db.set_habit_paused_until_date (habit['id'], paused_until_date_dt)
         if paused_until_date_dt:
             self.show_info_banner(self.top_win, '"' + habit['activity'] + '" paused until ' + self.dt_to_display_date(paused_until_date_dt))
         else:
@@ -403,6 +371,13 @@ class MainWindow:
         return dt.strftime('%Y-%m-%d')
 
 
+    def dt_to_day_number(self, dt):
+        day_num = int(dt.strftime('%w'))
+        if day_num == 0:
+            day_num = 7
+        return day_num
+
+
     def get_today_dt(self):
         today = datetime.date.today()
         return today
@@ -526,9 +501,12 @@ class MainWindow:
         button.connect('clicked', self.on_main_menu_go_to_date_click)
         menu.append(button)
 
+        # Stats page not yet implemented
+        """
         button = gtk.Button(_('Stats'))
         button.connect('clicked', self.on_main_menu_stats_click)
         menu.append(button)
+        """
 
         button = gtk.Button(_('Master Habits list'))
         button.connect('clicked', self.on_main_menu_master_habits_list_click)
@@ -599,15 +577,21 @@ class MainWindow:
         return self.get_generated_hildon_context_menu(menu_def)
 
 
-    def get_day_status_cmenu(self):
-        menu_def = [ \
+    def get_day_status_cmenu(self, show_open_timer = False):
+        menu_def = []
+        if show_open_timer:
+            menu_def.extend([ \
+                    ['Open Timer', self.on_status_cmenu_open_timer_selected], \
+            ])
+
+        menu_def.extend([ \
                 ['Done', self.on_status_cmenu_done_selected], \
                 ['75%', self.on_status_cmenu_75pct_selected], \
                 ['50%', self.on_status_cmenu_50pct_selected], \
                 ['25%', self.on_status_cmenu_25pct_selected], \
-                ['Missed', self.on_status_cmenu_missed_selected], \
                 ['Clear', self.on_status_cmenu_clear_selected], \
-        ]
+                ['Missed', self.on_status_cmenu_missed_selected], \
+        ])
 
         return self.get_generated_hildon_context_menu(menu_def)
 
@@ -667,9 +651,9 @@ class MainWindow:
         iter = buf.get_iter_at_offset(0)
         i_tag = buf.create_tag('i', style=pango.STYLE_ITALIC)
         b_tag = buf.create_tag('b', weight=pango.WEIGHT_BOLD)
-        buf.insert_with_tags(iter, 'HabitJewel v' + VERSION, b_tag)
+        buf.insert_with_tags(iter, APP_DISPLAY_NAME + ' v' + VERSION, b_tag)
         buf.insert(iter, ', copyright © Ashley Hooper, 2016\n\n')
-        buf.insert_with_tags(iter, 'HabitJewel', i_tag)
+        buf.insert_with_tags(iter, APP_DISPLAY_NAME, i_tag)
         buf.insert(iter, ' tracks your desired habits and their fulfillment on a regular \
 basis, helping to motivate your self-improvement.\n\n')
         buf.insert(iter, 'Habits are defined as follows:\n\n\
@@ -711,6 +695,43 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
 
 
 
+    ##################################
+    # General Habits-related functions
+    ##################################
+
+
+    def completion_pct_to_status_rgb_color(self, completion):
+        if completion > 0:
+            if completion > 100:
+                completion = 100
+            red = (100 - completion) * 0.01
+            green = (completion * 0.004) + 0.6
+            blue = 0.0
+        else:
+            red = 0.6
+            green = 0.6
+            blue = 0.6
+        return [red, green, blue]
+
+
+    def fraction_to_hex(self, fraction):
+        if fraction > 0:
+            return int(fraction * 256) - 1
+        else:
+            return 0
+
+
+    def completion_pct_to_status_hex_color(self, completion):
+        [red, green, blue] = self.completion_pct_to_status_rgb_color(completion)
+        red   = self.fraction_to_hex(red)
+        green = self.fraction_to_hex(green)
+        blue  = self.fraction_to_hex(blue)
+        return '#%02x%02x%02x' % (red, green, blue)
+
+
+
+
+
     ##############################
     # Master Habits List functions
     ##############################
@@ -763,7 +784,7 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
 
 
     def fetch_master_habits_list(self):
-        return habitjewel_db.get_habits_list_all(conn)
+        return self.db.get_habits_list_all()
 
 
     def add_columns_to_master_habits_list_tv(self, treeview):
@@ -816,10 +837,10 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
             else:
                 status = 'active'
 
-            activity_markup = '<b>' + item['activity'] + '</b> '
+            activity_markup = item['activity']
 
             if item['null_measure'] != 1:
-                activity_markup += str(item['target_desc']);
+                activity_markup += ' <small>' + str(item['target_desc']) + '</small>';
 
             icon_pixbuf = self.get_pixbuf_filename_for_master_status(status)
 
@@ -896,7 +917,7 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
             return
         else:
             new_activity = self.touched_habit['activity'] + ' (' + _('copy') + ')'
-            habitjewel_db.clone_habit(conn, self.touched_habit['id'], new_activity)
+            self.db.clone_habit(self.touched_habit['id'], new_activity)
             self.redraw_master_habits_list()
 
 
@@ -904,7 +925,7 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
         if not self.touched_habit:
             return
         else:
-            habitjewel_db.delete_habit(conn, self.touched_habit['id'])
+            self.db.delete_habit(self.touched_habit['id'])
             self.redraw_master_habits_list()
 
 
@@ -989,14 +1010,6 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
 
         self.pan_area.add(self.day_habits_list_tv)
 
-        """
-        if first_run:
-            vbox_first_run = gtk.VBox()
-            first_run_label = gtk.Label(_('No habits'))
-            vbox_first_run.pack_start(first_run_label) 
-            self.pan_area.add(vbox_first_run)
-        """
-
         self.vbox_outer.pack_start(self.vbox_nav, False)
         self.vbox_outer.pack_start(self.pan_area, True, True)
 
@@ -1007,7 +1020,6 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
         if not self.touched_tv_col_title:
             return
         if self.touched_tv_col_title == TV_DAY_COL_NAME_ACTIVITY:
-            # gobject.idle_add(self.popup_hildon_menu, self.h_tv_day_activity_cmenu)
             gobject.idle_add(self.popup_hildon_menu, self.h_tv_day_activity_cmenu)
         elif self.touched_tv_col_title == TV_DAY_COL_NAME_CHECKBOX:
             gobject.idle_add(self.popup_hildon_menu, self.h_tv_day_status_cmenu)
@@ -1027,51 +1039,129 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
             for habit in self.day_habits_list:
                 if habit['id'] == index:
                     self.touched_habit = habit
+                    today_dt = self.get_today_dt()
+
                     show_unpause = False
+                    show_open_timer = False
+
                     if habit['paused_until_date']:
                         paused_until_date_dt = self.db_date_to_dt(habit['paused_until_date'])
-                        today_dt = self.get_today_dt()
                         if paused_until_date_dt > today_dt:
                             show_unpause = True
-                    
+
+                    if habit['to_minutes'] and self.view_date_dt == today_dt:
+                        show_open_timer = True
+
+
+                    # Refresh the activity context menu to show the unpause option
                     if show_unpause:
                         self.h_tv_day_activity_cmenu = self.get_day_activity_cmenu(True, True)
                     else:
                         self.h_tv_day_activity_cmenu = self.get_day_activity_cmenu()
+                    
+
+                    # Refresh the status context menu to show the start timer option
+                    if show_open_timer:
+                        self.h_tv_day_status_cmenu = self.get_day_status_cmenu(True)
+                    else:
+                        self.h_tv_day_status_cmenu = self.get_day_status_cmenu()
                     break
         else:
             self.touched_habit = None
 
 
     def create_day_habits_list_model(self):
-        lstore = gtk.ListStore(int, str, gtk.gdk.Pixbuf, int, str)
+        lstore = gtk.ListStore(int, str, gtk.gdk.Pixbuf, gtk.gdk.Pixbuf, int, str)
         # add columns to the tree view
         return lstore
 
 
     def fetch_habits_list_for_date(self, view_date_dt):
-        return habitjewel_db.get_habits_list_for_date(conn, view_date_dt)
+        return self.db.get_habits_list_for_date(view_date_dt)
 
 
     def populate_day_habits_list_ls(self, day_habits_list):
+        day_num_of_week = self.dt_to_day_number(self.view_date_dt)
+
         for item in day_habits_list:
             lstore_iter = self.day_habits_list_model.append()
             checkbox_pixbuf = self.get_pixbuf_filename_for_completion_status(item['pct_complete'])
  
-            activity_markup = '<b>' + item['activity'] + '</b> '
+            activity_markup = item['activity']
 
             if item['null_measure'] != 1:
-                activity_markup += str(item['target_desc']);
+                activity_markup += ' <i><small>' + str(item['target_desc']) + '</small></i>';
 
             if item['weekly_quota'] > 1:
-                activity_markup += ' [' + str(item['progress']) + '/' + \
-                        str(item['weekly_quota']) + ']'
+                if item['completion_total'] > 0:
+                    # Calculate a percentage completion for the week up to the current view date
+                    progress = ((float(item['completion_total']) / float(day_num_of_week)) / float(item['weekly_quota'])) * day_num_of_week * 100
+                else:
+                    progress = 0
 
+                highlight_colour = self.completion_pct_to_status_hex_color(progress)
+                activity_markup += ' <span foreground="' + str(highlight_colour) + '"> <small>' + str(item['completion_total']) + '/' + \
+                        str(item['weekly_quota']) + ':' + str(day_num_of_week) + '</small> </span>'
+
+            # Render the last 7 days history graph for the habit
+            pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, \
+                    HABIT_RECENT_HIST_PIXBUF_SIZE, HABIT_RECENT_HIST_PIXBUF_SIZE)
+            pixbuf.fill(0x00000000)
+            pixmap, mask = pixbuf.render_pixmap_and_mask()
+
+            pixmap = gtk.gdk.Pixmap(None, HABIT_RECENT_HIST_PIXBUF_SIZE, \
+                    HABIT_RECENT_HIST_BAR_HEIGHT + 8, 16)
+            cr = pixmap.cairo_create()
+
+            # Graph previous 7 days' history for habit, [0:7] returns all but view date
+            #for day in sorted(item['history'][:-1]):
+            for index, day in enumerate(sorted(item['history'])[0:7]):
+                day_completion = item['history'][day]
+                # If we have history for this day...
+                if len(day_completion) > 0:
+
+                    # If missed show red bar, height of 1
+                    if day_completion[0] == 0:
+                        rgb = [0.9, 0.0, 0.0]
+                        bar_y = HABIT_RECENT_HIST_BAR_HEIGHT
+                        bar_h = 4
+
+                    # For all other results
+                    else:
+                        # If from preceding week, draw in grey
+                        if day_completion[1] < 0:
+                            rgb = [0.5, 0.5, 0.5]
+                        # Otherwise, based on completion status
+                        else:
+                            rgb = self.completion_pct_to_status_rgb_color(day_completion[0])
+
+                        bar_y = (100 - day_completion[0]) * HABIT_RECENT_HIST_BAR_HEIGHT / 100
+                        bar_h = day_completion[0] * HABIT_RECENT_HIST_BAR_HEIGHT / 100
+
+                # Draw a placeholder since we have no history for this day
+                else:
+                    rgb = self.completion_pct_to_status_rgb_color(-1)
+                    bar_y = HABIT_RECENT_HIST_BAR_HEIGHT - 1
+                    bar_h = 1
+
+                # Draw the bar
+                cr.set_source_rgb(rgb[0], rgb[1], rgb[2])
+                cr.rectangle(index * 7, bar_y + 4, 5, bar_h)
+                cr.fill()
+
+
+            # Turn pixmap back into pixbuf for Treeview column
+            pixbuf.get_from_drawable(pixmap, gtk.gdk.colormap_get_system(), \
+                    0, 0, 0, 4, -1, -1)
+
+            # Add the row to the list store
             self.day_habits_list_model.set(lstore_iter, \
                 TV_DAY_COL_NUM_ID, \
                     item['id'], \
                 TV_DAY_COL_NUM_ACTIVITY, \
                     activity_markup, \
+                TV_DAY_COL_NUM_HISTICON, \
+                    pixbuf, \
                 TV_DAY_COL_NUM_CHECKBOX, \
                     checkbox_pixbuf, \
                 TV_DAY_COL_NUM_PCT_COMPLETE, \
@@ -1103,19 +1193,25 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
         treeview.append_column(column)
 
         # column for activity
-        renderer = gtk.CellRendererText()
-        renderer.set_property('wrap-mode', gtk.WRAP_WORD)
-        renderer.set_property('wrap-width', self.tv_day_activity_wrap_width)
-        column = gtk.TreeViewColumn(TV_DAY_COL_NAME_ACTIVITY, renderer, markup=TV_DAY_COL_NUM_ACTIVITY)
+        rend = gtk.CellRendererText()
+        rend.set_property('wrap-mode', gtk.WRAP_WORD)
+        rend.set_property('wrap-width', self.tv_day_activity_wrap_width)
+        column = gtk.TreeViewColumn(TV_DAY_COL_NAME_ACTIVITY, rend, markup=TV_DAY_COL_NUM_ACTIVITY)
         column.set_property('expand', True)
-        #column.tap_and_hold_setup(self.day_habits_list_menu)
+        treeview.append_column(column)
+
+        # column for history
+        rend = gtk.CellRendererPixbuf()
+        column = gtk.TreeViewColumn(TV_DAY_COL_NAME_HISTICON, rend, pixbuf=TV_DAY_COL_NUM_HISTICON)
         treeview.append_column(column)
 
         # column for checkbox
+        # below was for single-clickable checkbox toggle
         #checkbox = CellRendererClickablePixbuf()
         #checkbox.connect('clicked', self.habit_toggled, treeview)
         checkbox = gtk.CellRendererPixbuf()
         column = gtk.TreeViewColumn(TV_DAY_COL_NAME_CHECKBOX, checkbox, pixbuf=TV_DAY_COL_NUM_CHECKBOX)
+        # Make the column a little wider to make it easier to press on
         column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
         column.set_fixed_width(80)
         treeview.append_column(column)
@@ -1131,6 +1227,7 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
         #treeview.append_column(column)
 
 
+    # This function is no longer used
     def habit_toggled(self, widget, row_num, treeview):
         # Toggle habit completion status (fulfilled / unfulfilled / unknown)
         model = treeview.get_model()
@@ -1175,11 +1272,11 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
         self.populate_day_habits_list_ls(self.day_habits_list)
         today = datetime.date.today()
 
-        # Hide the pause menu if not viewing the current day
-        if self.view_date_dt != today:
-            self.h_tv_day_activity_cmenu = self.get_day_activity_cmenu(True)
-        else:
+        # If viewing the current day, show the pause menu and timer option
+        if self.view_date_dt == today:
             self.h_tv_day_activity_cmenu = self.get_day_activity_cmenu()
+        else:
+            self.h_tv_day_activity_cmenu = self.get_day_activity_cmenu(True)
 
         # Hide checkbox column for dates in the future
         checkbox_col = self.day_habits_list_tv.get_column(TV_DAY_COL_NUM_CHECKBOX)
@@ -1254,9 +1351,20 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
 
 
     def set_habit_percent_complete (self, habit_id, view_date_dt, percent_complete):
-        habitjewel_db.set_habit_pct_complete (conn,
-                habit_id, view_date_dt, percent_complete
-            )
+        self.db.set_habit_pct_complete (habit_id, view_date_dt, percent_complete)
+
+
+    def on_status_cmenu_open_timer_selected (self, menu_item):
+        if not self.touched_habit:
+            return
+        else:
+            self.timer_habit = self.touched_habit
+            # Initialise the timer window
+            self.timer_win = self.get_stackable_window(_(WIN_TITLE_TIMER))
+            self.timer_container = self.get_timer_container()
+            self.timer_win.add(self.timer_container)
+            self.timer_win.connect('destroy', self.on_timer_window_destroy)
+            self.timer_win.show_all()
 
 
     def on_status_cmenu_pct_selected (self, pct_complete):
@@ -1295,15 +1403,181 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
 
 
 
+    ##################################
+    # Countdown timer window functions
+    ##################################
+
+
+    def get_timer_container(self):
+        habit = self.timer_habit
+        activity_tbl = gtk.Table(2, 2, False)
+        activity_tbl.set_row_spacings(5)
+        activity_tbl.set_col_spacings(5)
+
+        a_btn = hildon.Button(gtk.HILDON_SIZE_AUTO, \
+                hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        a_btn.set_label(_('Timer for:'))
+
+        a_entry = gtk.Label(habit['activity'])
+        a_entry.set_line_wrap(True)
+        activity_tbl.attach(a_btn, 0, 1, 0, 1)
+        activity_tbl.attach(a_entry, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL)
+
+        t_btn = hildon.Button(gtk.HILDON_SIZE_AUTO, \
+                hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        t_btn.set_label(_('Target:'))
+
+        if habit['pct_complete'] > 0:
+            status = str(100 - habit['pct_complete']) + '% ' + _('remaining')
+        else:
+            status = _('not started')
+        t_text = habit['target_desc'] + ' (' + status + ')'
+        t_entry = gtk.Label(t_text)
+        t_entry.set_line_wrap(True)
+        activity_tbl.attach(t_btn, 0, 1, 1, 2)
+        activity_tbl.attach(t_entry, 1, 2, 1, 2, gtk.EXPAND|gtk.FILL)
+
+        if self.last_orientation == LANDSCAPE:
+            box_controls = gtk.HBox(False)
+        else:
+            box_controls = gtk.VBox(True)
+
+        box_controls.set_spacing(30)
+
+        # When timer is running, display
+        if not 'running' in self.timer and not 'remain_secs' in self.timer:
+            # Set slider initial value (current remaining time in mins if existing timer)
+            # Calculate remaining time based on percent already complete and target
+            remain_mins = (100 - habit['pct_complete']) * \
+                    (habit['target'] * habit['to_minutes']) * 0.01
+            self.timer['remain_secs'] = remain_mins * 60
+
+        else:
+            remain_mins = self.timer['remain_secs'] * 0.01666666
+
+
+        self.timer_adj = gtk.Adjustment(remain_mins, 0, habit['target'] * habit['to_minutes'] * 2, 0, 1)
+        scale = gtk.HScale(self.timer_adj)
+        # Disable decimal places in horizontal scale
+        scale.set_digits(0)
+        scale.connect('value_changed', self.on_timer_time_changed)
+
+        vbox_scale = gtk.VBox(True)
+        scale_lbl = gtk.Label(_('Countdown Time (Minutes)'))
+        vbox_scale.pack_start(scale_lbl, True, True) 
+        vbox_scale.pack_start(scale, True, True)
+
+        box_controls.pack_start(vbox_scale, True, True) 
+
+        self.timer_start_stop_btn = hildon.Button(gtk.HILDON_SIZE_AUTO, \
+                hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+        self.timer_start_stop_btn_hbox = self.get_timer_start_stop_btn_hbox()
+        self.timer_start_stop_btn.add(self.timer_start_stop_btn_hbox)
+        self.timer_start_stop_btn.set_size_request(width=300, height=150)
+        self.timer_start_stop_btn.connect('clicked', self.on_timer_start_stop_btn_click)
+
+        box_controls.pack_start(self.timer_start_stop_btn, True, True)
+
+        vbox = gtk.VBox()
+        vbox.pack_start(activity_tbl, True, True)
+        vbox.pack_start(box_controls, True, False)
+
+        return vbox
+
+
+    def set_timer_adjustment_value(self, value):
+        self.timer_adj.set_value(value) 
+
+
+    def get_timer_start_stop_btn_hbox(self):
+        # HBox for start/stop timer button
+        hbox = gtk.HBox()
+        if 'running' in self.timer:
+            icon_filename = TIMER_STOP_PIXBUF_FILE
+            label_text = _('Stop Timer')
+        else:
+            icon_filename = TIMER_START_PIXBUF_FILE
+            label_text = _('Start Timer')
+        pixbuf = gtk.gdk.pixbuf_new_from_file(img_dir + icon_filename)
+        img = gtk.image_new_from_pixbuf(pixbuf)
+        label = gtk.Label(label_text)
+        hbox.pack_start(img)
+        hbox.pack_start(label)
+        return hbox
+
+
+    def set_timer_countdown_time(self, seconds):
+        self.timer['remain_secs'] = seconds
+
+
+    def on_timer_start_stop_btn_click(self, win):
+        self.on_timer_start_or_stop()
+
+
+    def on_timer_start_or_stop(self):
+        if 'running' in self.timer:
+            self.timer.pop('running', None)
+            self.timer.pop('remain_secs', None)
+            self.show_info_banner(self.top_win, _('Timer Stopped'))
+        else:
+            self.timer['running'] = True
+            gobject.timeout_add_seconds(TIMER_TIMEOUT_INTERVAL_SECS, self.timer_countdown)
+            self.show_info_banner(self.top_win, _('Timer Started'))
+
+        # Why doesn't this work? Start/Stop button doesn't redraw when clicked
+        self.timer_start_stop_btn.remove(self.timer_start_stop_btn_hbox)
+        self.timer_start_stop_btn_hbox = self.get_timer_start_stop_btn_hbox()
+        self.timer_start_stop_btn.add(self.timer_start_stop_btn_hbox)
+        self.timer_win.show_all()
+
+
+    def on_timer_time_changed(self, widget):
+        if not 'running' in self.timer:
+            self.set_timer_countdown_time(self.timer_adj.get_value() * 60)
+        else:
+            self.timer_adj.set_value(self.timer['remain_secs'] * 0.01666666)
+
+
+    def timer_countdown(self):
+        if not 'running' in self.timer:
+            return False
+
+        # Timer is running but still has time to go
+        elif self.timer['remain_secs'] > TIMER_TIMEOUT_INTERVAL_SECS:
+            self.timer['remain_secs'] -= TIMER_TIMEOUT_INTERVAL_SECS
+            new_min = abs(self.timer['remain_secs'] % 60) 
+            if new_min == 0 or new_min >= 60 - TIMER_TIMEOUT_INTERVAL_SECS:
+                self.set_timer_adjustment_value(self.timer['remain_secs'] * 0.01666666)
+            return True
+
+        # Timer is running and time's up!
+        else:
+            self.set_timer_adjustment_value(0)
+            self.on_timer_start_or_stop()
+            pl = gst.element_factory_make("playbin", "player")
+            pl.set_property('uri','file:///usr/share/sounds/ui-wake_up_tune.wav')
+            pl.set_state(gst.STATE_PLAYING)
+            return False
+
+
+    def on_timer_window_destroy(self, win):
+        # Clear data structures and widgets
+        if 'running' in self.timer:
+            self.on_timer_start_or_stop()
+            self.show_info_banner(self.top_win, _('Timer Cleared'))
+            self.timer = None
+
+
+
+
     ################################
     # Habit editing window functions
     ################################
 
 
     def habit_edit_window(self, habit):
-
         # Get categories
-        categories = habitjewel_db.get_categories_list(conn)
+        categories = self.db.get_categories_list()
 
         self.edit_win = hildon.StackableWindow()
         vbox = gtk.VBox()
@@ -1439,7 +1713,7 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
 
 
     def set_target_picker_visibility(self):
-        if habitjewel_db.is_null_measure(conn, self.editing_habit['measure_desc']):
+        if self.db.is_null_measure(self.editing_habit['measure_desc']):
             self.edit_habit_target_picker.hide();
         else:
             self.edit_habit_target_picker.show();
@@ -1516,7 +1790,7 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
 
 
     def create_measures_selector(self, selected_measure = None):
-        measures = habitjewel_db.get_measures_list(conn)
+        measures = self.db.get_measures_list()
         selector = hildon.TouchSelector(text = True)
         index = 0
         for measure in measures:
@@ -1533,7 +1807,7 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
         if self.editing_habit['measure_desc'] != NULL_MEASURE_DESC:
             self.editing_habit['null_measure'] = '0'
             measure_id, unit, plural = \
-                    habitjewel_db.get_measure(conn, self.editing_habit['measure_desc'])
+                    self.db.get_measure(self.editing_habit['measure_desc'])
             self.editing_habit['unit'] = unit
             self.editing_habit['plural'] = plural
         else:
@@ -1628,7 +1902,7 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
                     valid = False
 
         if valid:
-            habitjewel_db.save_habit(conn, self.editing_habit)
+            self.db.save_habit(self.editing_habit)
             self.show_info_banner(self.top_win, 'Habit "' + self.editing_habit['activity'] + '" saved')
             self.edit_win.destroy()
             self.redraw_day_habits_list()
@@ -1645,15 +1919,15 @@ etc. of all habits, whereas the daily habits view only shows habits for the curr
         self.editing_habit = None
 
 
-
-
-
 ###################
 # end of MainWindow 
 ###################
 
 
-if __name__ == "__main__":
+
+
+
+if __name__ == '__main__':
     MainWindow = MainWindow()
     #gtk.gdk.threads_enter()
     gtk.main()
